/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package org.dataone.tidy.concurrent;

import java.util.Date;
import java.util.Set;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.Callable;
import java.util.concurrent.ConcurrentSkipListSet;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.ReentrantLock;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.dataone.service.util.DateTimeMarshaller;
/**
 *
 * @author waltz
 */
public class TidyJobThreadPoolExecutor extends ThreadPoolExecutor {
    private static Log logger = LogFactory.getLog( TidyJobThreadPoolExecutor.class);

    private ConcurrentSkipListSet<TidyJob> failures = new ConcurrentSkipListSet<TidyJob>();
    

    private static boolean pause = false;
    private ReentrantLock pauseLock = new ReentrantLock();
    private Condition unpause = pauseLock.newCondition();
    
    public TidyJobThreadPoolExecutor(int corePoolSize, 
                            int maximumPoolSize, 
                            long keepAliveTime, 
                            TimeUnit unit, 
                            BlockingQueue<Runnable> workQueue) {
        super(corePoolSize, maximumPoolSize, keepAliveTime, unit, workQueue);
        
        
    }



    @Override
    protected <T> TidyFutureTask<T> newTaskFor(Callable<T> callable) {
        return new TidyFutureTask<T>((TidyJob) callable);

    }

    @Override
    protected void beforeExecute(Thread t, Runnable r) {
        pauseLock.lock();
        try {
          while (pause)
            unpause.await();
        } catch (InterruptedException ie) {
          t.interrupt();
        } finally {
          pauseLock.unlock();
        }
        TidyFutureTask<Boolean> task = (TidyFutureTask<Boolean>) r;

    }
    public void pause() {
      pauseLock.lock();
      logger.debug("Pausing Queue");
      try {
        pause = true;
      } finally {
        pauseLock.unlock();
      }
    }

    public void resume() {
      pauseLock.lock();
      logger.debug("Resuming Queue");
      try {
        pause = false;
        unpause.signalAll();
      } finally {
        pauseLock.unlock();
      }
    }

    public boolean isPaused() {
        return pause;
    }
    
    @Override
    protected void afterExecute(Runnable runnable, Throwable throwable) {
        if (throwable == null && runnable == null) {
          logger.error("Very bad");
          return;
        }
//        super.afterExecute(r, t); 
//        TidyFutureTask<Boolean> task = (TidyFutureTask<Boolean>) r;

        
        if (throwable == null && runnable != null) {
            Class runnableClass = runnable.getClass();      
            
            if ( runnable instanceof TidyFutureTask) {
                TidyFutureTask<Boolean> task = (TidyFutureTask<Boolean>) runnable;
               
                
                TidyJob job = task.getTidyJob();
                if ((job) == null)
                        throw new NullPointerException("Cannot determine job from TidyFutureTask");
                try {
                        task.get();
                    } catch (InterruptedException ie) {
                        logger.error("afterExecute: " + job.getPid().getValue() + " -InterruptedException: " + ie.getMessage());
                        logger.error(ie);
                        System.err.print("[" + DateTimeMarshaller.serializeDateToUTC(new Date()) + "] afterExecute: " + job.getPid().getValue() + " StackTrace: ");
                        ie.printStackTrace();
                        job.setFailure(ie);
                        
                         failures.add(job);
                    } catch (Exception e) {
                        logger.error("afterExecute: " + job.getPid().getValue() + ": " +e);
                        System.err.print("[" + DateTimeMarshaller.serializeDateToUTC(new Date()) + "] afterExecute: " + job.getPid().getValue() + " StackTrace: ");
                        e.printStackTrace();  
                        job.setFailure(e);
                         failures.add(job);
                    }
 
                logger.debug("job is of pid " + job.getPid().getValue());
            } 
        }

    }


    public TidyJob getFailure() {
        
       TidyJob tidyJob = failures.pollLast();

        return tidyJob;
    }
    public Set<TidyJob> getFailures() {
        
       return this.failures;

    }

}
