package org.dataone.tidy.merge;

import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.dataone.cn.dao.exceptions.DataAccessException;
import org.dataone.configuration.Settings;
import org.dataone.service.types.v1.Identifier;
import org.dataone.service.types.v1.NodeReference;
import org.dataone.service.types.v1.SystemMetadata;
import org.dataone.service.util.TypeMarshaller;
import org.dataone.tidy.dao.SystemMetadataTidyDao;
import org.jibx.runtime.JiBXException;

public class ResourceSystemMetadataTidyDaoImpl implements SystemMetadataTidyDao {

    private static final String ORC_NODE_ID = Settings.getConfiguration().getString("tidy.dao.orc.nodeid");
    private static final String UCSB_NODE_ID = Settings.getConfiguration().getString("tidy.dao.ucsb.nodeid");
    private static final String UNM_NODE_ID = Settings.getConfiguration().getString("tidy.dao.unm.nodeid");

	private SystemMetadata resultingSysmeta;
	private String cnUCSB;
	private String cnUNM;
	private String cnORC;

	public ResourceSystemMetadataTidyDaoImpl(String resourceUCSB, String resourceUNM, String resourceORC) {
		this.cnUCSB = resourceUCSB;
		this.cnUNM = resourceUNM;
		this.cnORC = resourceORC;
	}

	@Override
	public List<Identifier> listIdentifiers() throws DataAccessException {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public Map<NodeReference, SystemMetadata> getSystemMetadata(
			Identifier identifier) throws DataAccessException {
		Map<NodeReference, SystemMetadata> map = new LinkedHashMap<NodeReference,SystemMetadata>();
		NodeReference nr1 = new NodeReference();
		nr1.setValue(UCSB_NODE_ID);
		NodeReference nr2 = new NodeReference();
		nr2.setValue(UNM_NODE_ID);
		NodeReference nr3 = new NodeReference();
		nr3.setValue(ORC_NODE_ID);
		try {
			if (this.cnUCSB != null) map.put(nr1, getSysmetaResource(this.cnUCSB));
			if (this.cnUNM != null) map.put(nr2, getSysmetaResource(this.cnUNM));
			if (this.cnORC != null) {
				map.put(nr3, getSysmetaResource(this.cnORC));
			} else {
//				map.put(nr3, null);
			}
		} catch (IOException e) {
			throw new DataAccessException(e);
		} catch (InstantiationException e) {
			throw new DataAccessException(e);
		} catch (IllegalAccessException e) {
			throw new DataAccessException(e);
		} catch (JiBXException e) {
			throw new DataAccessException(e);
		}
		return map;
	}

    private SystemMetadata getSysmetaResource(String file) 
    throws IOException, InstantiationException, IllegalAccessException, JiBXException 
    {
    	return TypeMarshaller.unmarshalTypeFromStream(SystemMetadata.class,
    			this.getClass().getResourceAsStream(file));
    }

	
	@Override
	public void saveSystemMetadata(SystemMetadata systemMetadata)
			throws DataAccessException {
		this.resultingSysmeta = systemMetadata;

	}
	
	public SystemMetadata getSavedResultingSysmeta() {
		return this.resultingSysmeta;
	}

}
